settingsPromises = {}
local playerGUID
local startTime
local duration
local combatIndex = 0
local savedMana = 0
local clearcastFadeTime = 0

local promisesBuffs = {
  [191615] = 556, --ACE
  [191616] = 764, --TWO
  [191617] = 972, --THREE
  [191618] = 1181, --FOUR
  [191619] = 1388, --FIVE
  [191620] = 1596, --SIX
  [191621] = 1596, --SEVEN
  [191622] = 2221, --EIGHT
}

local promisesCounterFrame = CreateFrame("Frame", "promisesCounterFrame", UIParent)

promisesCounterFrame:SetSize(100, 26)
promisesCounterFrame:ClearAllPoints()
promisesCounterFrame:SetPoint("CENTER", UIParent)
promisesCounterFrame:SetBackdrop({
    bgFile   = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
    edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 12,
    insets = { left = 3, right = 3, top = 3, bottom = 3, },
})
promisesCounterFrame:SetBackdropColor(0, 0, 0, 0)
promisesCounterFrame:SetBackdropBorderColor(0, 0, 0, 0)

promisesCounterFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
promisesCounterFrame:RegisterEvent("PLAYER_REGEN_DISABLED");
promisesCounterFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
promisesCounterFrame:RegisterEvent("ADDON_LOADED")
promisesCounterFrame:RegisterEvent("SPELL_AURA_REMOVED")
promisesCounterFrame:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")


promisesCounterFrame:EnableMouse(true)
promisesCounterFrame:SetMovable(true)
promisesCounterFrame:RegisterForDrag("LeftButton")
promisesCounterFrame:SetScript("OnDragStart", promisesCounterFrame.StartMoving)
promisesCounterFrame:SetScript("OnDragStop",function(self)
 self:StopMovingOrSizing()
 settingsPromises.XPos = self:GetLeft()
 settingsPromises.YPos = self:GetBottom()
end)

promisesCounterFrame.frameText = promisesCounterFrame:CreateFontString(nil, "BACKGROUND", "PVPInfoTextFont")
promisesCounterFrame.frameText:SetAllPoints();
promisesCounterFrame.frameText:SetText(0)
promisesCounterFrame.frameText:SetPoint("CENTER", promisesCounterFrame, "CENTER", 4, 0)
promisesCounterFrame.frameText:SetFont("Fonts\\FRIZQT__.TTF", 16, "OUTLINE")
promisesCounterFrame.frameText:SetJustifyH("CENTER")
promisesCounterFrame.frameText:SetShadowOffset(1, -1)
promisesCounterFrame.frameText:SetTextColor(0.32, 0.52, 1)

function promisesCounterFrame:initFlags()
  if settingsPromises.movableFlag == nil then
    settingsPromises.movableFlag = true;
  end
  if settingsPromises.announceFlag == nil then
    settingsPromises.announceFlag = true;
  end
  if settingsPromises.hideFlag == nil then
    settingsPromises.hideFlag = false;
  end
end

function promisesCounterFrame:updateVisibility()
  if settingsPromises.hideFlag and not IsEquippedItem(128710) then
    self:Hide()
  else
    self:Show()
  end
end

local CostTip = CreateFrame('GameTooltip')
local CostText = CostTip:CreateFontString()
CostTip:AddFontStrings(CostTip:CreateFontString(), CostTip:CreateFontString())
CostTip:AddFontStrings(CostText, CostTip:CreateFontString())

function promisesCounterFrame:GetPowerCost(spellID)
  if not spellID then return end
	CostTip:SetOwner(WorldFrame, 'ANCHOR_NONE')	
	CostTip:SetSpellByID(spellID)
	return CostText:GetText()  
end

function promisesCounterFrame:onCombatEnter()
    startTime = GetTime()
    combatIndex = combatIndex + 1
    savedMana = 0
    promisesCounterFrame.frameText:SetText(savedMana)
end

function promisesCounterFrame:addSavedMana()    
  local buffs = {191615, 191616, 191617, 191618, 191619,191620,191621,191622}
  local idx = 1
  repeat
    local name, _, _, _, _, _, _, _, _, _, spellID = UnitAura("player", idx, nil, "PLAYER|HELPFUL|CANCELABLE")
    for i=1,8,1 do
       if spellID and spellID == buffs[i] then 
        savedMana = savedMana + promisesBuffs[buffs[i]]        
       end   
    end    
    idx = idx + 1
    until name == nil
    promisesCounterFrame.frameText:SetText(savedMana)
end

    
function promisesCounterFrame:onCombatExit()
  local total_time = GetTime() - startTime
  local hour = min(floor(total_time/360), 99)
  local minute = mod(total_time/60, 60)
  local second = mod(total_time, 60);
	if hour == 0 then
	  duration = format("%02d:%02d", minute, second)
	else
	  duration = format("%02d:%02d:%02d", hour, minute, second)
	end
  
  if savedMana > 0 and settingsPromises.announceFlag then
    local msg = format("Promises deck. Mana saved: %s Duration: %s", savedMana, duration)
    SendChatMessage(msg, "SAY", "Common")
  end
end

promisesCounterFrame:SetScript("OnEvent",
  function(self, event,  ...)
  if event == "ADDON_LOADED"  then
    playerGUID = UnitGUID("player")
    if settingsPromises.XPos then   
      promisesCounterFrame:ClearAllPoints()
      promisesCounterFrame:SetPoint("BOTTOMLEFT",settingsPromises.XPos,settingsPromises.YPos)      
    end    
    promisesCounterFrame:initFlags()
    if not settingsPromises.movableFlag then
      promisesCounterFrame:SetMovable(false)
    end
    promisesCounterFrame:updateVisibility()
  elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
    local _, eventType, _, sourceGUID, _, _, _, destGUID = ...
    local spellId = select(12,...)
    if sourceGUID == playerGUID and eventType == "SPELL_CAST_SUCCESS" then
    if spellId == 8936 or spellId == 2061 then
        if clearcastFadeTime == GetTime() then
          return
        end 
    end
    local costText = promisesCounterFrame:GetPowerCost(spellId)
    local cost = strmatch(costText, MANA_COST:gsub("%%s ", ""))    
    if cost~=nil then
      --costText = gsub(costText, '%D', '') + 0
       promisesCounterFrame:addSavedMana()
    end   
    elseif  sourceGUID == playerGUID and eventType == "SPELL_AURA_REMOVED" then
      if spellId == 16870 or spellId == 114255 then
        clearcastFadeTime = GetTime()
      end
  end
  elseif  event == "PLAYER_EQUIPMENT_CHANGED" then
    promisesCounterFrame:updateVisibility()
  elseif  event == "PLAYER_REGEN_DISABLED" then
    promisesCounterFrame:onCombatEnter()
  elseif  event == "PLAYER_REGEN_ENABLED" then
    promisesCounterFrame:onCombatExit()
  end
end)

local dropDown = CreateFrame("Frame", "PCContextMenu", promisesCounterFrame, "UIDropDownMenuTemplate")
UIDropDownMenu_Initialize(dropDown, function(self, level, menuList)
 local info = UIDropDownMenu_CreateInfo()
 if (level or 1) == 1 then
    info.text = "Block position"
    info.func = function() promisesCounterFrame:SetMovable(not promisesCounterFrame:IsMovable()) settingsPromises.movableFlag=promisesCounterFrame:IsMovable() end
    info.checked = function() return not promisesCounterFrame:IsMovable() end
    info.isNotRadio = true
    UIDropDownMenu_AddButton(info)
    info.text = "Announce mana saved"
    info.func = function() settingsPromises.announceFlag = not settingsPromises.announceFlag end
    info.checked = function() return settingsPromises.announceFlag end
    info.isNotRadio = true
    UIDropDownMenu_AddButton(info)
    info.text = "Hide without trinket"
    info.func = function() settingsPromises.hideFlag = not settingsPromises.hideFlag promisesCounterFrame:updateVisibility() end
    info.checked = function() return settingsPromises.hideFlag end
    UIDropDownMenu_AddButton(info, level)
 end
end)

promisesCounterFrame:SetScript("OnMouseDown", function(self, button)
	if button == "RightButton" then
		ToggleDropDownMenu(1, nil, dropDown, "cursor", 3, -3)
	end
end)
